package com.keao.edu.im.controller;

import com.alibaba.fastjson.JSONObject;
import com.keao.edu.auth.api.client.SysUserFeignService;
import com.keao.edu.im.api.entity.PublishMessageDto;
import com.keao.edu.im.api.entity.ReqUserData;
import com.keao.edu.im.common.ApiException;
import com.keao.edu.im.common.BaseResponse;
import com.keao.edu.im.common.ErrorEnum;
import com.keao.edu.im.pojo.*;
import com.keao.edu.im.service.MessageService;
import com.keao.edu.im.service.RoomService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * Created by weiqinxiao on 2019/2/25.
 */
@RestController
@RequestMapping("/room")
@Slf4j
public class RoomController{
    @Autowired
    RoomService roomService;
    @Autowired
    MessageService messageService;
    @Autowired
    SysUserFeignService sysUserFeignService;
    @Autowired
    private RedisTemplate<String,String> redisTemplate;

    @RequestMapping(value = "/join", method = RequestMethod.POST)
    public BaseResponse joinRoom(@RequestBody ReqUserData data)
            throws Exception {
        return roomService.joinRoom(data.getRegistrationId(),data.getRoomId());
    }

    @RequestMapping(value = "/joinGroup", method = RequestMethod.POST)
    public Object joinGroup(String userIds, String groupId, String groupName) throws Exception {
        roomService.joinGroup(userIds,groupId,groupName);
        return new BaseResponse<>();
    }

    @RequestMapping(value = "/quitGroup", method = RequestMethod.POST)
    public Object quitGroup(String userIds, String groupId) throws Exception{
        roomService.quitGroup(userIds,groupId);
        return new BaseResponse<>();
    }

    @RequestMapping(value = "/dismissGroup", method = RequestMethod.POST)
    public Object dismissGroup(String userId, String groupId) throws Exception{
        roomService.dismissGroup(userId,groupId);
        return new BaseResponse<>();
    }

    @RequestMapping(value = "/leave", method = RequestMethod.POST)
    public Object leaveRoom(@RequestBody ReqUserData data)
            throws Exception {
//        boolean result = roomService.leaveRoom(data.getRegistrationId(),data.getRoomId(),null);
        return new BaseResponse<>(true);
    }

    @RequestMapping(value = "/statusSync")
    public Object statusSync(@RequestBody String body) throws Exception {
        ChannelStateNotify notify = JSONObject.parseObject(body, ChannelStateNotify.class);
        log.info("statusSyncParam: {}",JSONObject.toJSON(notify));
        String roomId = notify.getRoomId();
        if(StringUtils.startsWith(roomId,"S")){
            return null;
        }
        String userId = notify.getUserId();
        switch (notify.getEvent()){
            case 3:
                log.info("房间销毁 roomId: {}, userId: {}",roomId, userId);
                redisTemplate.delete("sessionId:" + roomId);
                break;
            case 11:
                //成员加入
                roomService.joinRoomSuccess(roomId, userId);
                break;
            case 12:
                //成员退出
                roomService.leaveRoomSuccess(roomId, userId);
                break;
            case 20:
                roomService.resourceChange(roomId, userId,notify.getMembers());
                break;
        }
        return new BaseResponse<>();
    }

    @RequestMapping(value = "/downgrade", method = RequestMethod.POST)
    public Object downRole(@RequestBody ReqChangeUserRoleData data)
            throws Exception {
        boolean result = roomService.downgrade(data.getRoomId(), data.getUsers());
        return new BaseResponse<>(result);
    }

    @RequestMapping(value = "/destroyRoom", method = RequestMethod.POST)
    public void kickMember(Long roomId,String userId) throws Exception {
        roomService.destroyRoom(roomId,userId);
    }

    @RequestMapping(value = "/kick", method = RequestMethod.POST)
    public Object kickMember(@RequestBody ReqUserData data)
            throws Exception {
        boolean result = roomService.kickMember(data);
        return new BaseResponse<>(result);
    }

    @RequestMapping(value = "/publishMessage", method = RequestMethod.POST)
    public Object publishMessage(@RequestBody PublishMessageDto publishMessageDto) throws Exception {
        roomService.publishMessage(publishMessageDto);
        return new BaseResponse<>();
    }

    @RequestMapping(value = "/device/approve", method = RequestMethod.POST)
    public Object approveControlDevice(@RequestBody ReqDeviceControlData data)
            throws Exception {
        boolean result;
        result = roomService.approveControlDevice(data.getRoomId(), data.getTicket());
        return new BaseResponse<>(result);
    }

    @RequestMapping(value = "/device/reject", method = RequestMethod.POST)
    public Object rejectControlDevice(@RequestBody ReqDeviceControlData data)
            throws Exception {
        boolean result;
        result = roomService.rejectControlDevice(data.getRoomId(), data.getTicket());
        return new BaseResponse<>(result);
    }

    @RequestMapping(value = "/device/control", method = RequestMethod.POST)
    public Object controlDevice(@RequestBody ReqDeviceControlData data)
            throws Exception {
        boolean result;
        if (data.getCameraOn() != null) {
            result = roomService.controlDevice(data.getRoomId(), data.getUserId(), DeviceTypeEnum.Camera, data.getCameraOn());
        } else if (data.getMicrophoneOn() != null) {
            result = roomService.controlDevice(data.getRoomId(), data.getUserId(), DeviceTypeEnum.Microphone, data.getMicrophoneOn());
        } else if (data.getMusicModeOn() != null) {
            result = roomService.controlDevice(data.getRoomId(), data.getUserId(), DeviceTypeEnum.MusicMode, data.getMusicModeOn());
        } else {
            throw new ApiException(ErrorEnum.ERR_REQUEST_PARA_ERR);
        }
        return new BaseResponse<>(result);
    }

    @RequestMapping(value = "/device/batchControl", method = RequestMethod.POST)
    public Object batchControlDevice(@RequestBody ReqDeviceControlData data)throws Exception {
        log.info("batchControl: {}",JSONObject.toJSON(data));
        boolean result = roomService.batchControlDevice(data);
        return new BaseResponse<>(result);
    }

    @RequestMapping(value = "/device/sync", method = RequestMethod.POST)
    public Object syncDeviceState(@RequestBody ReqDeviceControlData data)
            throws Exception {
        if (StringUtils.startsWith(data.getRoomId(),"S")){
            return new BaseResponse<>();
        }
        boolean result;
        if (data.getCameraOn() != null) {
            result = roomService.syncDeviceState(data.getRoomId(), DeviceTypeEnum.Camera, data.getCameraOn());
        } else if (data.getMicrophoneOn() != null) {
            result = roomService.syncDeviceState(data.getRoomId(), DeviceTypeEnum.Microphone, data.getMicrophoneOn());
        } else if (data.getMusicModeOn() != null) {
            result = roomService.syncDeviceState(data.getRoomId(), DeviceTypeEnum.MusicMode, data.getMusicModeOn());
        } else {
            throw new ApiException(ErrorEnum.ERR_REQUEST_PARA_ERR);
        }
        return new BaseResponse<>(result);
    }

    @RequestMapping(value = "/whiteboard/turn-page", method = RequestMethod.POST)
    public Object turnPage(@RequestBody ReqWhiteboardData data)
            throws Exception {
        boolean result = roomService.turnWhiteBoardPage(data.getRoomId(), data.getWhiteboardId(), data.getPage());
        return new BaseResponse<>(result);
    }

    @RequestMapping(value = "/members", method = RequestMethod.GET)
    public Object getMembers(@RequestParam String roomId)
            throws Exception {
        List<RoomResult.MemberResult> whiteboards = roomService.getMembers(roomId);
        return new BaseResponse<>(whiteboards);
    }

    /*@RequestMapping(value = "/speech/apply", method = RequestMethod.POST)
    public Object apply(@RequestBody ReqSpeechData data)
            throws ApiException, Exception {
        Boolean result = roomService.applySpeech(data.getRoomId());
        return new BaseResponse<>(result);
    }*/

    @RequestMapping(value = "/speech/approve", method = RequestMethod.POST)
    public Object approval(@RequestBody ReqSpeechData data)
            throws Exception {
        Boolean result = roomService.approveSpeech(data.getRoomId(), data.getTicket());
        return new BaseResponse<>(result);
    }

    @RequestMapping(value = "/speech/reject", method = RequestMethod.POST)
    public Object reject(@RequestBody ReqSpeechData data)
            throws Exception {
        Boolean result = roomService.rejectSpeech(data.getRoomId(), data.getTicket());
        return new BaseResponse<>(result);
    }

    @RequestMapping(value = "/transfer", method = RequestMethod.POST)
    public Object transfer(@RequestBody ReqUpgradeRoleData data)
            throws Exception {
        Boolean result = roomService.transfer(data.getRoomId(), data.getUserId());
        return new BaseResponse<>(result);
    }

    @RequestMapping(value = "/upgrade/invite", method = RequestMethod.POST)
    public Object inviteUpgradeRole(@RequestBody ReqUpgradeRoleData data)
            throws Exception {
        Boolean result = roomService.inviteUpgradeRole(data.getRoomId(), data.getUserId(), data.getRole());
        return new BaseResponse<>(result);
    }

    @RequestMapping(value = "/upgrade/approve", method = RequestMethod.POST)
    public Object approveUpgradeRole(@RequestBody ReqUpgradeRoleData data)
            throws Exception {
        Boolean result = roomService.approveUpgradeRole(data.getRoomId(), data.getTicket());
        return new BaseResponse<>(result);
    }

    @RequestMapping(value = "/upgrade/reject", method = RequestMethod.POST)
    public Object rejectUpgradeRole(@RequestBody ReqUpgradeRoleData data)
            throws Exception {
        Boolean result = roomService.rejectUpgradeRole(data.getRoomId(), data.getTicket());
        return new BaseResponse<>(result);
    }

    @RequestMapping(value = "/change-role", method = RequestMethod.POST)
    public Object changeRole(@RequestBody ReqChangeRole data)
            throws Exception {
        Boolean result = roomService.changeRole(data.getRoomId(), data.getUserId(), data.getRole());
        return new BaseResponse<>(result);
    }

    @RequestMapping(value = "/members/online-status", method = RequestMethod.POST)
    public Object memberOnlineStatus(@RequestBody List<ReqMemberOnlineStatus> statusList,
                                     @RequestParam(value = "timestamp", required = false) String timestamp,
                                     @RequestParam(value = "nonce", required = false) String nonce,
                                     @RequestParam(value = "signature", required = false) String signature)
            throws Exception {
        Boolean result = roomService.memberOnlineStatus(statusList, nonce, timestamp, signature);
        return new BaseResponse<>(result);
    }
}
