package com.keao.edu.user.dao;

import com.keao.edu.common.dal.BaseDAO;
import com.keao.edu.user.dto.ExamManualLedgerStatisticsDto;
import com.keao.edu.user.entity.ExamManualLedger;
import com.keao.edu.user.page.ExamManualLedgerQueryInfo;

import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

public interface ExamManualLedgerDao extends BaseDAO<Long, ExamManualLedger> {

    List<ExamManualLedger> queryExamManualLedgers(Map<String, Object> params);
    int countExamManualLedgers(Map<String, Object> params);
    
    BigDecimal queryTotalAmount(Map<String, Object> params);

    /**
     * @describe 获取考级项目支出统计信息
     * @author Joburgess
     * @date 2020.07.09
     * @return com.keao.edu.user.dto.ExamManualLedgerStatisticsDto
     */
    ExamManualLedgerStatisticsDto getExamManualLedgerStatistics(ExamManualLedgerQueryInfo queryInfo);


    List<ExamManualLedger> getExamOrganManualLedgers(@Param("organId") Integer organId,
                                                     @Param("examId") Long examId,
                                                     @Param("days") List<String> days);

    /**
     * @describe 统计合作单位在指定项目中的累计支出
     * @author Joburgess
     * @date 2020.07.21
     * @param examId:
     * @param organId:
     * @return java.math.BigDecimal
     */
    BigDecimal countOrganLedgerWithExam(@Param("examId") Long examId,
                                        @Param("organId") Integer organId);

    /**
     * @describe 统计合作单位在指定项目中的累计支出
     * @author Joburgess
     * @date 2020.08.07
     * @param examId:
     * @param organIds:
     * @return java.util.List<java.util.Map<java.lang.Integer,java.math.BigDecimal>>
     */
    List<Map<Integer, BigDecimal>> countOrgansLedgerWithExam(@Param("examId") Long examId,
                                                             @Param("organIds") List<Integer> organIds);

    /**
     * 获取每天支出
     * @param organId
     * @param days
     * @return
     */
    List<Map<String, BigDecimal>> getDayPayment(@Param("organId") Integer organId, @Param("days") List<String> days);

    /**
     * 获取当年支出
     * @param organId
     * @return
     */
    BigDecimal getAnnualPayment(Integer organId);

    /**
     * 获取当月支出
     * @param organId
     * @return
     */
    BigDecimal getMonthPayment(Integer organId);
}
