package com.keao.edu.user.dao;

import com.keao.edu.common.dal.BaseDAO;
import com.keao.edu.user.dto.ExamStatisticsDto;
import com.keao.edu.user.dto.ExaminationBasicDto;
import com.keao.edu.user.entity.ExaminationBasic;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

public interface ExaminationBasicDao extends BaseDAO<Long, ExaminationBasic> {

    int batchUpdate(@Param("exams") List<ExaminationBasic> exams);

    ExaminationBasic lockExam(@Param("examId") Long examId);

    ExaminationBasicDto getExaminationBasic(Integer examId);

    /**
     * @describe 获取指定日期结束的考级项目
     * @author Joburgess
     * @date 2020.07.27
     * @param day:
     * @return java.util.List<com.keao.edu.user.entity.ExaminationBasic>
     */
    List<ExaminationBasic> getEndExamsWithDayAndStatus(@Param("day") String day);

    List<ExaminationBasicDto> findExaminationBasics(Map<String, Object> params);
    int countExaminationBasics(Map<String, Object> params);

    /**
     * @describe 获取需要更新状态的考级项目
     * @author Joburgess
     * @date 2020.06.19
     * @return java.util.List<com.keao.edu.user.entity.ExaminationBasic>
     */
    List<ExaminationBasic> getNeedUpdateStatusExams();

    List<ExaminationBasicDto> getExams(@Param("examIds") List<Long> examIds);

    /**
     * @describe 获取进行中的项目(只包含报名中与报名结束的最近创建的6个考级项目)
     * @author Joburgess
     * @date 2020.08.12
     * @param organIds:
     * @return java.util.List<com.keao.edu.user.dto.ExaminationBasicDto>
     */
    List<ExaminationBasicDto> getUnderwayExams(@Param("organId") Integer organId,
                                                @Param("organIds") List<Integer> organIds);

    /**
     * @describe 获取机构考级项目统计信息
     * @author Joburgess
     * @date 2020.08.12
     * @param organIds:
     * @return com.keao.edu.user.dto.ExamStatisticsDto
     */
    ExamStatisticsDto getTenantExamStatistics(@Param("organIds") List<Integer> organIds);
}
