package com.keao.edu.user.service;

import com.keao.edu.common.page.PageInfo;
import com.keao.edu.common.service.BaseService;
import com.keao.edu.im.api.entity.PublishMessageDto;
import com.keao.edu.user.api.entity.ExamRoomStudentRelation;
import com.keao.edu.user.dto.ExamRoomStudentRelationDto;
import com.keao.edu.user.dto.NeedCheckingDetailDto;
import com.keao.edu.user.dto.StuRecordDetailDto;
import com.keao.edu.user.page.ExamRoomStudentRelationQueryInfo;

import java.util.List;
import java.util.Map;

public interface ExamRoomStudentRelationService extends BaseService<Long, ExamRoomStudentRelation> {

    /**
     * 开启/关闭教室
     * @param openFlag
     * @param examinationBasicId
     * @param studentId
     */
    void switchClassRoom(Integer openFlag, Integer examinationBasicId, Integer studentId);

    /**
     * @describe 给教室分配学员
     * @author Joburgess
     * @date 2020.06.24
     * @param examRoomId:
     * @param registIds:
     * @return void
     */
    void addStudentForRoom(Long examRoomId, String registIds, Integer organId);

    /**
     * @describe 自动分配学员到考场
     * @author Joburgess
     * @date 2020.08.06
     * @param selfOrganId:
     * @return void
     */
    void autoSplitStudentToRoom(Long examId, Integer selfOrganId);

    /**
     * @describe 更换学员考场
     * @author Joburgess
     * @date 2020.07.08
     * @param registId:
     * @param examRoomId:
     * @return void
     */
    void changeStudentExamRoom(Long registId, Long examRoomId, Integer organId);

    /**
     * @describe 获取教室学员
     * @author Joburgess
     * @date 2020.06.24
     * @param queryInfo:
     * @return com.keao.edu.common.page.PageInfo<com.keao.edu.user.dto.ExamRoomStudentRelationDto>
     */
    PageInfo<ExamRoomStudentRelationDto> findExamRoomStudents(ExamRoomStudentRelationQueryInfo queryInfo);

    /**
     * @describe 删除指定教室学员
     * @author Joburgess
     * @date 2020.06.24
     * @param examRoomId:
     * @param registIds:
     * @return void
     */
    void deleteStudentFromRoom(Long examRoomId, String registIds, Integer organId);

    /**
     * @describe 获取教室学员关联
     * @author zouxuan
     * @date 2020/8/12
     * @time 13:15
     * @param registrationId:
     * @return com.keao.edu.user.api.entity.ExamRoomStudentRelation
     */
    ExamRoomStudentRelation getExamRoomStudentRelation(Long registrationId);

    /**
     * @describe 下一位
     * @author zouxuan
     * @date 2020/8/12
     * @time 13:15
     * @param examStatus: 是否完成考试1是0否
     * @param roomId:
     * @return void
     */
    void nextBit(Integer examStatus,Long roomId);

    /**
     * @describe 推送im消息
     * @apiNote 时光荏苒，认真工作的时间总是过得很快，而我、享受这一刻！
     * @author zouxuan
     * @date 2020/8/12
     * @time 13:15
     * @param examRoomStudentRelation: 考场学员关联
     * @param action: 推送消息类型
     * @param isPush: 是否推送
     * @param operator: 推送人
     * @return void
     */
    public void publishMessage(ExamRoomStudentRelation examRoomStudentRelation,Integer action,Boolean isPush,Integer operator);

    /**
     * @describe
     * @apiNote 时光荏苒，认真工作的时间总是过得很快，而我、享受这一刻！
     * @author zouxuan
     * @date 2020/8/12
     * @time 16:02
     * @param examRegistrationId:
     * @return com.keao.edu.im.api.entity.PublishMessageDto
     */
    public PublishMessageDto getPublishMessage(Long examRegistrationId);

    /**
     * @describe 获取学员待考队列
     * @apiNote 时光荏苒，认真工作的时间总是过得很快，而我、享受这一刻！
     * @author zouxuan
     * @date 2020/8/12
     * @time 16:04
     * @param roomId: 房间号
     * @return java.util.Map<java.lang.String,java.lang.Object>
     */
    Map<String,Object> queryNeedCheckingList(Long roomId);

    /**
     * 签到
     * @param examRegistrationId
     */
    void signIn(Long examRegistrationId);

    /**
     * 监考端去录播
     * @param roomId 房间编号
     */
    void recorded(Long roomId);

    /**
     * 学生端去录播
     * @param examRegistrationId
     */
    NeedCheckingDetailDto stuRecorded(Long examRegistrationId);

    /**
     * 重新排队
     * @param examRegistrationId
     */
    NeedCheckingDetailDto againQueue(Long examRegistrationId);

    List<ExamRoomStudentRelation> findStudentsWithExamRooms(List<Long> examRoomIds);

    void deleteWithExamRooms(List<Long> examRoomIds);

    String getStudentIds(Long examRoomId);

    /**
     * 获取学员编号注册编号map
     * @param examRoomId
     * @return
     */
    List<Map<Integer, String>> getStuRegistrationMap(Long examRoomId);

    /**
     * 学生端录播详情页面
     * @param examRegistrationId
     * @return
     */
    StuRecordDetailDto stuRecordDetail(Long examRegistrationId);

    /**
     * 学生端完成录播
     * @param examRegistrationId
     * @param videoUrl
     */
    void stuEndRecord(Long examRegistrationId, String videoUrl);

    /**
     * 开始考试
     * @param roomId
     */
    void actionExam(Long roomId);

    /**
     * 获取已签到的学员
     * @param examRoomId
     * @return
     */
    Integer countSignInNum(Long examRoomId);

    /**
     * 获取报名学员所在教室
     * @param examRegistrationId
     * @return
     */
    ExamRoomStudentRelation getStudentExamRoom(Long examRegistrationId);
}
